package MuGeN::CompAnalResultSourceWindow::OrigamiResultsWindow; 

use strict;

use Data::Dumper;

use MuGeN::CompAnalResultSourceWindow;
use MuGeN::Resources::Origami::OrigamiHandler;

@MuGeN::CompAnalResultSourceWindow::OrigamiResultsWindow::ISA=qw(Gtk2::Dialog MuGeN::CompAnalResultSourceWindow);

my $false = 0;
my $true = 1;

sub new{
    my $class=shift;
    my $manager=shift;

    my $self=new Gtk2::Dialog();

    bless $self,$class;

    $self->signal_connect('delete_event', sub {return 1});
    $self->signal_connect('destroy_event', sub {return 1});
    $self->set_title('Results from Origami');
    $self->set_modal( $true );
    $self->set_default_size( 400, 400 );
    $self->set_position('center');
	
    $self->{manager}=$manager;

    $self->_load_tree_data();

    $self->_create_organism_tree();
    $self->_create_result_tree();

  
      
    my $notebook =Gtk2::Notebook->new();
    $self->vbox->pack_start($notebook,$true,$true,0);

    my $tree_frame=Gtk2::Frame->new();
    my $tree_window =Gtk2::ScrolledWindow->new( undef, undef );
    $tree_window->set_policy( 'automatic', 'automatic' );
    $tree_window ->add_with_viewport( $self->{organism_tree} );
    $tree_window ->show();
    $tree_frame->add($tree_window);
    $tree_frame ->show();
    $notebook->append_page( $tree_frame ,Gtk2::Label->new("Origami by Organisms" ) );

    my $tree_frame=Gtk2::Frame->new();
    my $tree_window =Gtk2::ScrolledWindow->new( undef, undef );
    $tree_window->set_policy( 'automatic', 'automatic' );   
    $tree_window ->add_with_viewport(  $self->{result_tree} );
    $tree_window ->show();
    $tree_frame->add($tree_window);
    $tree_frame ->show();
    $notebook->append_page( $tree_frame ,Gtk2::Label->new("Origami by Results" ) );
    
    $notebook->set_current_page( 0 );
    $notebook->show();
    
    my $action_area=$self->action_area();
    
    my $bouton_ok=Gtk2::Button->new( "OK" );
    $bouton_ok->show();
    $bouton_ok->signal_connect('clicked',sub {$self->_load_results()});
    $action_area->pack_start($bouton_ok,$false,$false,0);
    $self->{'button_ok'} =  $bouton_ok;
    
    my $bouton_cancel =Gtk2::Button->new("Cancel");
    $bouton_cancel->show();
    $bouton_cancel->signal_connect( 'clicked',sub{$self->hide();return 1;});
    $action_area->pack_start($bouton_cancel,$false,$false,0);

    return $self;  	
}

sub _load_tree_data {
    
    my $self=shift;

    $self->{organisms}=MuGeN::Resources::Origami::OrigamiHandler::getAllOrganisms();
    $self->{organismsbyid}={};
    foreach my $organism (@{$self->{organisms}}) {
	$self->{organismsbyid}->{$organism->{id}}=$organism;
    }
    
    $self->{elements}=MuGeN::Resources::Origami::OrigamiHandler::getElementsOfOrganisms($self->{organisms});
    $self->{elementsbyid}={};
    foreach my $element (@{$self->{elements}}) {
	$self->{elementsbyid}->{$element->{id}}=$element;
    }
    
    $self->{resulttypes}=MuGeN::Resources::Origami::OrigamiHandler::getResultTypes();	
    $self->{resulttypesbyid}={};
    foreach my $resulttype (@{$self->{resulttypes}}) {
	$self->{resulttypesbyid}->{$resulttype->{id}}=$resulttype;
    }

    my $results=MuGeN::Resources::Origami::OrigamiHandler::getResultsOfElements($self->{elements});
    my @tmptotalresults=();
    foreach my $result (@$results) {
	push @tmptotalresults,{result => $result,
			       organism => $self->{organismsbyid}->
			       {$result->{organismId}},
				   element => $self->{elementsbyid}->
			       {$result->{elementId}},
			       };
    }

    @{$self->{totalresults}}=
	sort {$a->{organism}->{name} cmp $b->{organism}->{name}}
    @tmptotalresults;

    return 1;  
}

sub _create_organism_tree {
    
    my $self = shift;

    my $orgtreemodel=Gtk2::TreeStore->new("Glib::String");
    my $orgtreeview=Gtk2::TreeView->new_with_model($orgtreemodel);
    $orgtreeview->insert_column_with_attributes(0,"Organism",Gtk2::CellRendererText->new(),"text",0);
    $self->{organism_tree}=$orgtreeview;
    $self->{organism_tree}->show();  

    $self->{resultsbyorganisms}={};
    my %orgpathsbyid=();

    foreach my $organism (sort {$a->{name} cmp $b->{name}} 
			  @{$self->{organisms}}) {

	my $iter=$orgtreemodel->append(undef);
	$orgtreemodel->set($iter,0,$organism->{name});
	$orgpathsbyid{$organism->{id}}=$orgtreemodel->get_path($iter);
    }

    my %eltpathsbyid=();

    foreach my $element (sort {$a->{type}." ".$a->{description} cmp
				   $b->{type}." ".$b->{description}}
			 @{$self->{elements}}) {

	my $orgpath=$orgpathsbyid{$element->{organismId}};
	my $orgiter=$orgtreemodel->get_iter($orgpath);
	my $eltiter=$orgtreemodel->append($orgiter);
	$orgtreemodel->set($eltiter,0,$element->{type}." ".$element->{description});
	$eltpathsbyid{$element->{id}}=$orgtreemodel->get_path($eltiter);
    }


    foreach my $totalresult (@{$self->{totalresults}}) {
	my $result=$totalresult->{result};
	my $resulttype=$self->{resulttypesbyid}->{$result->{typeId}};

	my $eltpath=$eltpathsbyid{$totalresult->{element}->{id}};
	my $eltiter=$orgtreemodel->get_iter($eltpath);
	my $resiter=$orgtreemodel->append($eltiter);
	$orgtreemodel->set($resiter,0,$resulttype->{description});
	my $string=$orgtreemodel->get_string_from_iter($resiter);
	$self->{resultsbyorganisms}->{$string}=$totalresult;

    }
    $orgtreeview->get_selection()->signal_connect('changed', sub { $self->_result_selected_from_organisms(@_)});

}


sub _create_result_tree {
    
    my $self = shift;

    my $restreemodel=Gtk2::TreeStore->new("Glib::String");
    my $restreeview=Gtk2::TreeView->new_with_model($restreemodel);
    $restreeview->insert_column_with_attributes(0,"Result type",Gtk2::CellRendererText->new(),"text",0);
    $self->{result_tree}=$restreeview;
    $self->{result_tree}->show();  
    $self->{resultsbyresults}={};

    my %resultinfobyid=();

    foreach my $resulttype (@{$self->{resulttypes}}) {

	my $iter=$restreemodel->append(undef);
	$restreemodel->set($iter,0,$resulttype->{description});
	$resultinfobyid{$resulttype->{id}}={ -path => $restreemodel->get_path($iter),
					     -organisms => {}};
    }


    foreach my $totalresult (sort {

	my $orga=$a->{organism};
	my $elta=$a->{element};
	my $stringa=$orga->{name}." ".$elta->{type}." ".$elta->{description};

	my $orgb=$b->{organism};
	my $eltb=$b->{element};
	my $stringb=$orgb->{name}." ".$eltb->{type}." ".$eltb->{description};

	return $stringa cmp $stringb

    }@{$self->{totalresults}}) {
	my $result=$totalresult->{result};
	my $organism=$totalresult->{organism};
	my $element=$totalresult->{element};

	my $resultinfo=$resultinfobyid{$result->{typeId}};
	my $organisms=$resultinfo->{-organisms};
	if (!defined $organisms->{$organism->{id}}) {
	    my $typeiter=$restreemodel->get_iter($resultinfo->{-path});
	    my $orgiter=$restreemodel->append($typeiter);
	    $restreemodel->set($orgiter,0,$organism->{name});
	    $organisms->{$organism->{id}}={-path => $restreemodel->get_path($orgiter),
				           -elements => {}};
	}

	my $elements=$organisms->{-elements};
	if (!defined $elements->{$element->{id}}) {
	    my $orgiter=$restreemodel->get_iter($organisms->{$element->{organismId}}->{-path});
	    my $eltiter=$restreemodel->append($orgiter);
	    $restreemodel->set($eltiter,0,$element->{type}." ".$element->{description});
	    my $string=$restreemodel->get_string_from_iter($eltiter);
	    $self->{resultsbyresults}->{$string}=$totalresult;
	}


    }
    $restreeview->get_selection()->signal_connect('changed', sub { $self->_result_selected_from_results(@_)});
}


sub _result_selected_from_organisms {
    my $self=shift;
    my $selection=shift;

    if ($selection->count_selected_rows()) {
	my ($model,$iter)=$selection->get_selected();
	my $string=$model->get_string_from_iter($iter);
	if (defined $self->{resultsbyorganisms}->{$string}) {
	    my $totalresult=$self->{resultsbyorganisms}->{$string};
	    $self->_result_selected($totalresult,$model,$iter,$self->{resultsbyorganisms});
	}
    }
    
}

sub _result_selected_from_results {
    my $self=shift;
    my $selection=shift;

    if ($selection->count_selected_rows()) {
	my ($model,$iter)=$selection->get_selected();
	my $string=$model->get_string_from_iter($iter);


	if (defined $self->{resultsbyresults}->{$string}) {
	    my $totalresult=$self->{resultsbyresults}->{$string};
	    $self->_result_selected($totalresult,$model,$iter,$self->{resultsbyresults});
	}
    }
}

sub _result_selected {
    my $self=shift;
    my $totalresult=shift;
    my $model=shift;
    my $iter=shift;
    my $stringhash=shift;

    my $result=$totalresult->{result};
    my $resulttype=$self->{resulttypesbyid}->{$result->{typeId}};

    my $description=$resulttype->{description};

    if (($description =~ /Gene Context Conservation/ ||
	$description =~ /Orthology Map/ ||
	$description =~ /Genome Subtraction/) &&
	!defined $totalresult->{selement}) {
	$self->_load_related_elements($totalresult,$model,$iter,$stringhash);

    } else {
	$self->{selectedresult}=$totalresult;
    }

    return 1;
}


sub _load_results  {
    my $self=shift;
    my $totalresult=$self->{selectedresult};

    my $mapinfo=$self->{manager}->get_mapinfo('NAME' => $self->{mapname});
    if (defined $mapinfo) {
	my $source = new MuGeN::CompAnalResultSource::OrigamiResultSource($totalresult,$self->{manager}->{resultlogwindow});
	$self->{manager}->add_results_from_source($self->{mapname},$source);
    }

    $self->hide();

    return 1;
}

sub _load_related_elements {
    my $self=shift;
    my $totalresult=shift;
    my $model=shift;
    my $iter=shift;
    my $stringhash=shift;

    my $selements = MuGeN::Resources::Origami::OrigamiHandler::getRelatedElementsOfResult($totalresult->{result});
	    
    foreach my $selement (sort {
	my $asorganism=$self->{organismsbyid}->{$a->{organismId}};
	my $astring=$asorganism->{name}." / ".$a->{type}." ".$a->{description};
	my $bsorganism=$self->{organismsbyid}->{$b->{organismId}};
	my $bstring=$bsorganism->{name}." / ".$b->{type}." ".$b->{description};
	return $astring cmp $bstring;
    }@$selements){
	my $newtotalresult={};
	foreach my $key (keys %$totalresult) {
	    $newtotalresult->{$key}=$totalresult->{$key};
	}
	my $sorganism=$self->{organismsbyid}->{$selement->{organismId}};
	my $subiter=$model->append($iter);
	$model->set($subiter,0,$sorganism->{name}." / ".$selement->{type}." ".$selement->{description});
	$newtotalresult->{selement}=$selement;
	$newtotalresult->{sorganismname}=$self->{organismsbyid}->{$selement->{organismId}}->{name};
	my $string=$model->get_string_from_iter($subiter);
	$stringhash->{$string}=$newtotalresult;
    }
}

1;

