package MuGeN::FeatureDataSource::SmallgenFeatureDataSource;

use strict;

use DBI;
use DBD::Pg;
use Bio::Seq;
use Bio::SeqFeature::Generic;

use MuGeN::FeatureDataSource;
use MuGeN::PreferencesHandler;

@MuGeN::FeatureDataSource::SmallgenFeatureDataSource::ISA=qw(MuGeN::FeatureDataSource);


sub set_preferences {
    my $mugen=$MuGeN::PreferencesHandler::Preferences->getDocumentElement();
    my @nodes=$mugen->getElementsByTagName("featuredatasources");
    my $fds=shift @nodes;
    my @nodes=$fds->getElementsByTagName("smallgen",0);
    if (scalar(@nodes)==0) {
	my $doctype=$MuGeN::PreferencesHandler::Preferences->getDoctype();
	$doctype->addElementDecl("smallgen","EMPTY");
	$doctype->addAttDef("smallgen","host","CDATA","#REQUIRED");
	$doctype->addAttDef("smallgen","dbname","CDATA","#REQUIRED");
	$doctype->addAttDef("smallgen","user","CDATA","#REQUIRED");
	$doctype->addAttDef("smallgen","passwd","CDATA","#REQUIRED");
	my $smallgen=$MuGeN::PreferencesHandler::Preferences->createElement("smallgen");
	$smallgen->setAttribute("host","127.0.0.1");
	$smallgen->setAttribute("dbname","smallgenes");
	$smallgen->setAttribute("user","unknown");
	$smallgen->setAttribute("passwd","unknown");
	$fds->appendChild($smallgen);
    }
}

BEGIN {
    MuGeN::PreferencesHandler::register_listener(\&set_preferences);
}

my $dbh=undef;

my %descriptions=();
my %accessnumbers=();
my $entries_retrieved=0;

sub init {

    my $res=0;
    if (!$dbh) {
	my @nodes=$MuGeN::PreferencesHandler::Preferences->getElementsByTagName("smallgen");
	my $smallgen=shift @nodes;
	if ($smallgen) {
	    my $host=$smallgen->getAttribute("host");
	    my $dbname=$smallgen->getAttribute("dbname");
	    my $user=$smallgen->getAttribute("user");
	    my $passwd=$smallgen->getAttribute("passwd");
	    $dbh=DBI->connect("dbi:Pg:host=$host;dbname=$dbname",$user,$passwd);
	}
    }
    $res=1
	if (defined $dbh);

    return $res;
}


sub _retrieve_by_entry_id {
    my $self=shift;
    my $entry_id=shift;
    my $start=shift;
    my $end=shift;
    my $qualifiers=shift;               # non renseign

    my %features_gb=();
    my %features_show=();
    my $sth=undef;
    my $sth2=undef;
    my $rv=undef;
    my $rowhash=undef;
    my $ref_hash=undef;
    my $compteur_gb=0;
    my $compteur_show =0;

    my $location_restriction='';
    $location_restriction="and (posmin>=$start and posmax<=$end)" 
	if ($start>0 && $end>$start);

    # Entres Genbank
    my $sth=$dbh->prepare("select gb_id, type_feat, posmin, posmax, strand 
                             from gb_feature
                             where contig_id = $entry_id $location_restriction
                              order by posmin");

    $sth->execute;
    while ($rowhash=$sth->fetchrow_hashref()) {
	$compteur_gb++;
	$features_gb{$compteur_gb}={
	    id=>$rowhash->{gb_id},
	    primary=>$rowhash->{type_feat},
	    start=>$rowhash->{posmin},
	    end=>$rowhash->{posmax},
	    strand=>$rowhash->{strand},
	    tags => {}
	};

        my $sth2=$dbh->prepare("select type_qual, value_qual
                             from gb_qualifier
                             where gb_id = $rowhash->{gb_id}");
	
	$sth2->execute();
	while($ref_hash = $sth2->fetchrow_hashref){ 	
	    $features_gb{$compteur_gb}->{tags}->{$ref_hash->{type_qual}}=$ref_hash->{value_qual};
	}

	$sth2->finish();
    }

    $sth->finish();

    # Entres CDS Show 
    my $sth=$dbh->prepare("select putative_cds.putcds_id, posmin, posmax, strand
                             from putative_cds, putative_cds_ref as p1
                             where putative_cds.contig_id = $entry_id $location_restriction
                               and p1.putcds_id = putative_cds.putcds_id
                               and p1.show_res_id <> 0
                               and p1.putcds_id not in (select putcds_id
                                                          from putative_cds_ref
                                                         where putative_cds_ref.putcds_id = putative_cds.putcds_id
                                                           and putative_cds_ref.gb_id <> 0)
                               order by posmin");
                              
    $sth->execute;
    while ($rowhash=$sth->fetchrow_hashref()) {
	$compteur_show++;
	$features_show{$compteur_show}={
	    id=>$rowhash->{putcds_id},
	    primary=>'CDS',
	    start=>$rowhash->{posmin},
	    end=>$rowhash->{posmax},
	    strand=>$rowhash->{strand},
	    tags => {}
	};
    }

    $sth->finish();

    my $seq='';
    my $sequences='' ;
    $sth=$dbh->prepare("select dna_sequence from contig where contig_id = $entry_id");
    $rv=$sth->execute;
    $rowhash=$sth->fetchrow_hashref();
    $seq=$rowhash->{dna_sequence} ;


    my $accessnumber='';
    my $rows=$dbh->selectall_arrayref("SELECT contig_ac FROM contig WHERE contig_id=$entry_id");
    $accessnumber=$rows->[0]->[0];
    $seq=new Bio::Seq(-seq => $seq,
		      -id => 'smallgen_retrieved_sequence',
		      -accession_number => $accessnumber);
    
    my $indice_show=1;
    foreach my $feature_id (sort ({$a<=>$b} keys (%features_gb))) {

	my $start=$features_gb{$feature_id}->{start};
	my $end=$features_gb{$feature_id}->{end};
	my $strand=$features_gb{$feature_id}->{strand};
	my $primary=$features_gb{$feature_id}->{primary};

	while ($start > $features_show{$indice_show}->{start} && $indice_show <= $compteur_show) {
	    my $seqfeature=new Bio::SeqFeature::Generic(
						    -start => $features_show{$indice_show}->{start},
						    -end => $features_show{$indice_show}->{end},
						    -strand => $features_show{$indice_show}->{strand} ,
						    -primary => $features_show{$indice_show}->{primary},
						    -tag => $features_show{$indice_show}->{tags});
	    $seq->add_SeqFeature($seqfeature);  
	    $indice_show++;
	}

	my $seqfeature=new Bio::SeqFeature::Generic(
						    -start => $start,
						    -end => $end,
						    -strand => $strand,
						    -primary => $primary,
						    -tag => $features_gb{$feature_id}->{tags});
	$seq->add_SeqFeature($seqfeature);
    }

    # Cas gnomes non rfrencs dans Genbank
    if ($compteur_gb==0){
	 foreach my $feature_id (sort ({$a<=>$b} keys (%features_show))) {
	     my $start=$features_show{$feature_id}->{start};
	     my $end=$features_show{$feature_id}->{end};
	     my $strand=$features_show{$feature_id}->{strand};
	     my $primary=$features_show{$feature_id}->{primary};
	     my $seqfeature=new Bio::SeqFeature::Generic(
						    -start => $start,
						    -end => $end,
						    -strand => $strand,
						    -primary => $primary,
						    -tag => $features_show{$feature_id}->{tags});
	     $seq->add_SeqFeature($seqfeature);
	 }
    }

    return $seq;

}

sub _retrieve_by_description {
    my $self=shift;
    my $description=shift;
    my $start=shift;
    my $end=shift;
    my $qualifiers=shift;

    my $entry_id=$descriptions{$description};
    return undef
	if (!defined $entry_id);

    return $self->_retrieve_by_entry_id($entry_id,$start,$end,$qualifiers);
}

sub _retrieve_by_accessnumber {
    my $self=shift;
    my $accessnumber=shift;
    my $start=shift;
    my $end=shift;
    my $qualifiers=shift;

    my $entry_id=$accessnumber;
    return undef
	if (!defined $entry_id);

    return $self->_retrieve_by_entry_id($entry_id,$start,$end,$qualifiers);
}


sub _retrieve_entry_descriptions {

    my $sth=$dbh->prepare("SELECT contig_id,contig_ac,organism 
                             FROM contig, genome
                             WHERE genome.genome_id = contig.genome_id");
    my $rv=$sth->execute();
    while (my $rowhash=$sth->fetchrow_hashref()) {
	my $entry_id=$rowhash->{contig_id};
	my $description=$rowhash->{organism};
	my $accessnumber=$rowhash->{contig_id};
	$descriptions{$description}=$entry_id;
    }
    $entries_retrieved=1;
}

sub get_entry_descriptions {

    _retrieve_entry_descriptions
	if (!$entries_retrieved);

    return sort keys %descriptions;

}

sub get_access_number {
    my $description=shift;

    _retrieve_entry_descriptions
	if (!$entries_retrieved);

    my $entry_id=$descriptions{$description};
    return $entry_id;
}

sub new {
    my $class=shift;
    my %params=();

    $params{start}=-1;
    $params{end}=-1;
    $params{qualifiers}=0;
    while (my $paramname=shift) {
	$params{lc $paramname}=shift;
    }

    my $self=_new MuGeN::FeatureDataSource;

    bless $self,$class;

    my $seqobj=undef;

    if (!defined $dbh) {
	warn "Trying to use Smallgenes as data source without valid database connection.\n";
	return undef;
    }

    _retrieve_entry_descriptions
	if (!$entries_retrieved);


    if (defined $params{description}) {
	$seqobj=$self->_retrieve_by_description($params{description},
						$params{start},
						$params{end},
						$params{qualifiers});
    } elsif (defined $params{accessnumber}){

	$seqobj=$self->_retrieve_by_entry_id($params{accessnumber},
						 $params{start},
						 $params{end},
						 $params{qualifiers});
    }

    $self->MuGeN::FeatureDataSource::_load_features($seqobj)
	if (defined $seqobj);

    return $self;
}

END {
    $dbh->disconnect()
	if (defined $dbh);
}

1
