package MuGeN::Gui::DataSourceConfigWindow;

use strict;

use Gtk;

use MuGeN::PreferencesHandler;

@MuGeN::Gui::DataSourceConfigWindow::ISA=qw(Gtk2::Dialog);

sub new {
    
    my $class=shift;
    my $manager=shift;

    my $self=Gtk2::Dialog->new();
    bless $self,$class;

    $self->set_title('Data Source Configuration');
    $self->set_default_size(480,320);

    $self->{manager}=$manager;

    my $vbox=$self->vbox();
    my $scrolledwindow=Gtk2::ScrolledWindow->new(undef,undef);
    $scrolledwindow->show();
    $vbox->pack_start($scrolledwindow,1,1,1);

    my $innervbox=Gtk2::VBox->new();
    $innervbox->show();
    $scrolledwindow->add_with_viewport($innervbox);
    


    $self->{datasources}={};
    my $mugen=$MuGeN::PreferencesHandler::Preferences->getDocumentElement();
    my ($featuredatasource,@dummy)=$mugen->getElementsByTagName('featuredatasources');
    if (defined $featuredatasource) {
	my @children=$featuredatasource->getChildNodes();
	foreach my $child (@children) {
	    if ($child->getNodeType ==  XML::DOM::ELEMENT_NODE) {
		my $sourcename=$child->getNodeName();
		my $frame=Gtk2::Frame->new(ucfirst $sourcename);
		$frame->show();
		$innervbox->pack_start($frame,1,1,1);
		my $framevbox=Gtk2::VBox->new();
		$framevbox->show();
		$frame->add($framevbox);
		$self->{datasources}->{$sourcename}={};
		foreach my $attribute ($child->getAttributes()->getValues) {
		    my $attname=$attribute->getNodeName();
		    my $attval=$attribute->getNodeValue();
		    $self->{datasources}->{$sourcename}->{"$attname"}=$attval;
		    $self->{datasources}->{$sourcename}->{"$attname.node"}=$attribute;
		    my $atthbox=Gtk2::HBox->new();
		    $atthbox->show();
		    $framevbox->pack_start($atthbox,1,1,1);
		    my $attnamelabel=Gtk2::Label->new(ucfirst $attname);
		    $attnamelabel->show();
		    $atthbox->pack_start($attnamelabel,1,1,1);
		    if ($attname ne 'enabled') {
			my $attvalueentry=Gtk2::Entry->new_with_max_length(128);
			$attvalueentry->set_text($attval);
			$attvalueentry->set_visibility(0)
			    if ($attname eq 'passwd');
			$self->{datasources}->{$sourcename}->{"$attname.widget"}=$attvalueentry;
			$attvalueentry->show();
			$atthbox->pack_start($attvalueentry,1,1,1);
		    } else {
			my $yesbox=Gtk2::RadioButton->new(undef,'yes');
			$self->{datasources}->{$sourcename}->{"yesbox"}=$yesbox;
			$yesbox->show();
			$atthbox->pack_start($yesbox,1,1,1);
			my $nobox=Gtk2::RadioButton->new($yesbox,'no');
			$self->{datasources}->{$sourcename}->{"nobox"}=$yesbox;
			$nobox->show();
			$atthbox->pack_start($nobox,1,1,1);
			if ($attval eq 'yes') {
			    $yesbox->set_active(1);
			    $nobox->set_active(0);
			} else {
			    $yesbox->set_active(0);
			    $nobox->set_active(1);
			}
		    }
		}
	    }
	}
    }


    my $actionarea=$self->action_area();
    my $applybtn=Gtk2::Button->new('Apply');
    $applybtn->show();
    $applybtn->signal_connect('clicked',sub { $self->hide(); $self->apply_changes() });
    $actionarea->pack_start($applybtn,1,1,1);
    my $cancelbtn=Gtk2::Button->new('Cancel');
    $cancelbtn->show();
    $cancelbtn->signal_connect('clicked',sub { $self->hide();} );
    $actionarea->pack_start($cancelbtn,1,1,1);

    return $self;
    
}


sub apply_changes {
    
    my $self=shift;


    foreach my $datasourcename (keys %{$self->{datasources}}) {
	my $datasource=$self->{datasources}->{$datasourcename};
	if ($datasource->{'yesbox'}->get_active()) {
	    $datasource->{'enabled.node'}->setValue('yes');
	} else {
	    $datasource->{'enabled.node'}->setValue('no');
	}
	
	foreach my $key (keys %{$datasource}) {
	    if (defined $datasource->{"$key.node"} &&
		defined $datasource->{"$key.widget"}) {
		$datasource->{"$key.node"}->setValue($datasource->{"$key.widget"}->get_text());
	    }
	}
    }
    $self->{manager}->rescan_datasources();
}

